@@ -6,13 +6,14 @@
 ## This code is free software; you can redistribute it and/or modify
 ## it under the same terms as Perl itself.
 ##
-## $Id: Agent.pm,v 1.62 2002/07/25 22:37:11 chad Exp $
+## $Id: Agent.pm,v 1.73 2003/05/07 00:00:38 vipul Exp $
 
 package Razor2::Client::Agent;
 
 use lib qw(lib);
 use strict;
 use Getopt::Long; 
+use IO::File;
 
 use base qw(Razor2::Client::Core);
 use base qw(Razor2::Client::Config);
@@ -20,19 +21,22 @@ use base qw(Razor2::Logger);
 use base qw(Razor2::String); 
 use Razor2::Preproc::Manager;
 use Razor2::Preproc::VR1;
+use Data::Dumper;
 use vars qw( $VERSION $PROTOCOL );
 
 
-
 $PROTOCOL = $Razor2::Client::Version::PROTOCOL;
 $VERSION  = $Razor2::Client::Version::VERSION;
 
 
 
-
 sub new {
     my ($class, $breed) = @_;
 
+    # For Taint Friendliness
+    delete $ENV{PATH};
+    delete $ENV{BASH_ENV};
+
     my @valid_program_names = qw(
             razor-check 
             razor-report  
@@ -93,12 +97,12 @@ sub do_conf {
     if ($self->{opt}->{create_conf}) {
         $self->{force_discovery} = 1;
         $self->{force_bootstrap_discovery} = 1;
-        $self->log(8," -create will force discvery for all .lst");
+        $self->log(8," -create will force complete discovery");
     }
     if ($self->{opt}->{force_discovery}) {
         $self->{force_discovery} = 1;
         $self->{force_bootstrap_discovery} = 1;
-        $self->log(8," -discover will force discvery for all .lst");
+        $self->log(8," -discover will force complete discovery");
     }
     if ($self->{opt}->{debug} && !$self->{opt}->{debuglevel}) {
         $self->{conf}->{debuglevel} = 9 if $self->{conf}->{debuglevel} < 9;
@@ -120,7 +124,7 @@ sub do_conf {
                         DontDie       => 1,
                         Log2FileDir   => defined($self->{conf}->{tmp_dir}) ? $self->{conf}->{tmp_dir} : "/tmp",
                      );
-        $self->{logref} = $logger;
+        $self->{logref} = ref($logger) ? $logger : 0;
         # log error strings at loglevel 11.  Pick a high number 'cuz
         # if its really an error, it will be in errstr for caller
         $self->{logerrors} = 11; 
@@ -132,7 +136,7 @@ sub do_conf {
     #$self->logobj(3,"warn on this $chadrules");
 
     # creates razorhome, and sets $self->{razorhome} if successful
-    return $self->errprefix("Could not create stuff") unless $self->create_home_conf(); 
+    return $self->errprefix("Could not create 'razorhome'") unless $self->create_home_conf(); 
     $self->compute_identity;
 
     $self->log(5,"computed razorhome=$self->{razorhome}, conf=$self->{razorconf}, ident=$self->{identity}");
@@ -142,6 +146,7 @@ sub do_conf {
 # if a debug log statement requires extra work, check this call before doing it.
 sub logll {
     my ($self, $loglevel) = @_;
+    return unless $self->{logref};
     return 1 if ($self->{logref}->{LogDebugLevel} >= $loglevel);
     return;
 }
@@ -223,15 +228,16 @@ sub log2file {
 
 sub doit {
     my $self = shift;
+    my $args = shift;
     my $r;
 
     $self->log(2," $self->{name_version} starting razor-$self->{breed} $self->{args}");
     $self->log(9,"uname -a: ". `uname -a`) if $self->logll(9);
 
-    $r = $self->checkit    if $self->{breed} eq 'check';
-    $r = $self->adminit    if $self->{breed} eq 'admin';
-    $r = $self->reportit   if $self->{breed} eq 'report';
-    $r = $self->reportit   if $self->{breed} eq 'revoke';
+    $r = $self->checkit($args)    if $self->{breed} eq 'check';
+    $r = $self->adminit($args)    if $self->{breed} eq 'admin';
+    $r = $self->reportit($args)   if $self->{breed} eq 'report';
+    $r = $self->reportit($args)   if $self->{breed} eq 'revoke';
 
     # return exit code
     # 0, 1 => ok
@@ -418,13 +424,16 @@ sub read_options {
 # returns 1 if no match (legit)
 # returns 2 if error
 sub checkit {
+
     my $self = shift;
+    my $args = shift;
 
     # check for spam.
     # input can be one of 
     #   file - single mail
     #   mbox - many  mail
     #   sig  - 1 or more sigs
+    #   or a filehandle provided via args
         
     my $objects;
     if ($self->{conf}->{sig}) {
@@ -440,14 +449,16 @@ sub checkit {
                 eng => $self->{conf}->{sigengine},
                 sig => $sig,
             };
+            $hr->{ep4} = "7542-10";
             $hr->{ep4} = $self->{conf}->{sigep4} if $self->{conf}->{sigep4};
             push @sigs, $hr;
         }
         $self->log (5,"received ". (scalar @sigs) ." valid cmd-line sigs.");
         $objects = $self->prepare_objects(\@sigs) or return 2;
     } else {
+        
+        my $mails = $self->parse_mbox($args) or return 2;
 
-        my $mails = $self->parse_mbox() or return 2;
         $objects  = $self->prepare_objects($mails) or return 2;
         #
         # if mail is whitelisted, its not spam.
@@ -503,12 +514,11 @@ sub checkit {
         return 1;
             
     }
+
     return 1 if $self->{opt}->{printhash};
 
-
-
     # only check good objects
-    my @goodones;
+    my @goodones;                 # this should be optimized! 
     foreach my $obj (@$objects) {
         next if $obj->{skipme};
         push @goodones, $obj;
@@ -537,6 +547,9 @@ sub checkit {
     my $has_spam = 0;
     foreach my $obj (@$objects) {
 
+        $obj->{spam} = 0 if $obj->{skipme};
+        $obj->{spam} = 0 unless defined $obj->{spam};
+
         if ($obj->{spam} > 0) {
             return 0 if $only1check;
             $has_spam = 1;
@@ -567,7 +580,7 @@ sub checkit {
 # returns 2 if error
 sub adminit { 
     my $self = shift;
-   
+
     my $done_something = 0;
 
     if ($self->{opt}->{create_conf}) {
@@ -651,14 +664,28 @@ sub registerit {
 # returns 0 if success
 # returns 2 if error
 sub reportit {
-    my $self = shift;
+
+    my ($self, $args) = @_;
 
     my $ident = $self->get_ident;
     unless ($ident) {
-        $self->errprefix("No identity, Did you register ?  more");
+        $self->errprefix("Bootstrap Error: Your Razor2 identity was not found.\n   " . 
+                         "  If you haven't registered, please do so:\n" .
+                         "     \"razor-admin -register -user=[name/email_address] -pass=[password]\".\n". 
+                         "     (Further information can be found in the razor-admin(1) manpage)\n" .
+                         "  If you did register, please ensure your identity symlink (or file) is in order.\nmore");
+
         return 2;
     }
 
+    # background myself 
+    unless ($self->{opt}->{foreground}) {
+        chdir '/'; 
+        fork && return 0; 
+        POSIX::setsid;
+        # close 0, 1, 2;
+    }
+
     if ($self->{opt}->{authen_only}) { 
         $self->authenticate($ident) or return;
         $self->log(5,"Done - authenticate only.");
@@ -666,15 +693,7 @@ sub reportit {
         return 2;
     }
 
-    my $mails   = $self->parse_mbox() or return 2;
-
-    # background myself 
-    unless ($self->{opt}->{foreground}) {
-        chdir '/'; 
-        fork && return 0; 
-        POSIX::setsid;
-        # close 0, 1, 2;
-    }
+    my $mails   = $self->parse_mbox($args) or return 2;
 
     my $objects = $self->prepare_objects($mails) or return 2;  
 
@@ -720,27 +739,52 @@ sub reportit {
 
 
 sub parse_mbox {
-    my ($self) = @_;
+    my ($self, $args) = @_;
 
     my @mails;
     my @message;
+    my $passed_fh = 0;
+    my $aref;
     
     # There are different kinds of mbox formats, we just split on simplest case.
     # djb defines mbox, mboxrd, mboxcl, mboxcl2 
     # http://www.qmail.org/qmail-manual-html/man5/mbox.html
     #
     # non-mbox support added, thanx to Aaron Hopkins <aaron@die.net>
-    push @ARGV, "-" unless scalar @ARGV;
+
+    if (exists $$args{"fh"}) { 
+        @ARGV = ();
+        push @ARGV, $$args{'fh'};
+        $passed_fh = 1;
+    } elsif (exists $$args{"aref"}) { 
+       $aref = $$args{"aref"};
+    } elsif (!scalar @ARGV) { 
+        push @ARGV, "-"
+    }
+
+    if ($$args{'aref'}) { 
+        my @foo = (\join'', @{$$args{'aref'}});
+        return \@foo;
+    }
+
     foreach my $file (@ARGV) {
+        my $fh = new IO::File;
         my @message = ();
-        open IN, "<$file" or return $self->error("Can't open $file: $!");
-        next unless defined(my $line = <IN>);
+        if (ref $file) { 
+            $fh = $file
+        } else { 
+            open $fh, "<$file" or return $self->error("Can't open $file: $!");
+        }
+
+        my $line = <$fh>;
+        next unless $line;
+
         if ($line =~ /^From /) {
             $self->log(8,"reading  mbox formatted mail from ". 
                 ($file eq '-' ? "<stdin>" : $file));
             while (1) {
                 push @message, $line;
-                $line = <IN>;
+                $line = <$fh>;
                 if (!defined($line) || $line =~ /^From /) {
                     push @mails, \join ('', @message);
                     @message = ();
@@ -750,9 +794,9 @@ sub parse_mbox {
         } else {
             $self->log(8,"reading straight RFC822 mail from ". 
                 ($file eq '-' ? "<stdin>" : $file));
-            push @mails, \join ('', map {s/^(>*From )/>$1/; $_} $line,<IN>);
+            push @mails, \join ('', map {s/^(>*From )/>$1/; $_} $line, <$fh>);
         }
-        close IN;
+        close $fh unless $passed_fh;
     }
 
     my $cnt = scalar @mails; 
@@ -983,5 +1027,4 @@ sub writeservers {
 }
 
 
-
 1;
